@@ -1,6 +1,7 @@
 import sys
 import os
 import argparse
+import itertools
 import socket
 
 from chaussette import logger as chaussette_logger
@@ -9,7 +10,7 @@ from chaussette.backend import get, backends
 
 
 def make_server(app, host=None, port=None, backend='wsgiref', backlog=2048,
-                logger=None, address_family=socket.AF_INET,
+                spawn=None, logger=None, address_family=socket.AF_INET,
                 socket_type=socket.SOCK_STREAM):
     logger = logger or chaussette_logger
     logger.info('Application is %r' % app)
@@ -20,9 +21,23 @@ def make_server(app, host=None, port=None, backend='wsgiref', backlog=2048,
 
     server_class = get(backend)
     logger.info('Using %r as a backend' % server_class)
-    server = server_class((host, port), app, backlog=backlog,
-                          address_family=address_family,
-                          socket_type=socket_type)
+    if spawn:
+        logger.info('Spawning method: %r' % spawn)
+    server_class_kwargs = {
+        'backlog': backlog,
+        'address_family': address_family,
+        'socket_type': socket_type,
+    }
+    if spawn is not None:
+        server_class_kwargs['spawn'] = spawn
+
+    try:
+        server = server_class((host, port), app, **server_class_kwargs)
+    except TypeError:
+        logger.exception("Failed to create backend %s, you might be trying "
+                         "to use --spawn on a backend that does "
+                         "not support it" % backend)
+        raise
     return server
 
 
@@ -44,7 +59,7 @@ _NO_UNIX = ('waitress', 'fastgevent', 'eventlet')
 
 
 def serve_paste(app, global_conf, **kw):
-    port = int(kw.get(kw['port'], 8080))
+    port = int(kw.get('port', 8080))
     host = kw.get('host', '127.0.0.1')
     backlog = int(kw.get('backlog', 2048))
     backend = kw.get('backend', 'wsgiref')
@@ -94,9 +109,9 @@ def main():
     parser = argparse.ArgumentParser(description='Run some watchers.')
     parser.add_argument('--port', type=int, default=8080)
     parser.add_argument('--address-family', type=str, default='AF_INET',
-                        choices=_ADDRESS_FAMILY.keys())
+                        choices=sorted(_ADDRESS_FAMILY.keys()))
     parser.add_argument('--socket-type', type=str, default='SOCK_STREAM',
-                        choices=_SOCKET_TYPE.keys())
+                        choices=sorted(_SOCKET_TYPE.keys()))
     group = parser.add_mutually_exclusive_group()
     group.add_argument('--host', default='localhost')
     group.add_argument('--fd', type=int, default=-1)
@@ -104,15 +119,21 @@ def main():
     group.add_argument('--backlog', type=int, default=2048)
     parser.add_argument('--backend', type=str, default='wsgiref',
                         choices=backends())
+    parser.add_argument('--use-reloader', action='store_true',
+                        help="Restart server when source files change")
+    parser.add_argument('--spawn', type=int, default=None,
+                        help="Spawn type, only makes sense if the backend "
+                             "supports it (gevent)")
     parser.add_argument('application', default='chaussette.util.hello_app',
                         nargs='?')
     parser.add_argument('--pre-hook', type=str, default=None)
     parser.add_argument('--post-hook', type=str, default=None)
     parser.add_argument('--python-path', type=str, default=None)
 
-    log_levels = LOG_LEVELS.keys() + [key.upper() for key in LOG_LEVELS.keys()]
+    log_levels = itertools.chain.from_iterable((key.upper(), key)
+                                               for key in LOG_LEVELS.keys())
     parser.add_argument('--log-level', dest='loglevel', default='info',
-                        choices=log_levels, help="log level")
+                        choices=sorted(log_levels), help="log level")
 
     parser.add_argument('--log-output', dest='logoutput', default='-',
                         help="log output")
@@ -151,20 +172,33 @@ def main():
         logger.info('Sorry %r does not support unix sockets' % args.backend)
         sys.exit(0)
 
-    try:
-        httpd = make_server(app, host=host, port=args.port,
-                            backend=args.backend, backlog=args.backlog,
-                            logger=logger,
-                            address_family=address_family,
-                            socket_type=_SOCKET_TYPE[args.socket_type])
+    def inner():
         try:
-            httpd.serve_forever()
-        except KeyboardInterrupt:
+            httpd = make_server(app, host=host, port=args.port,
+                                backend=args.backend, backlog=args.backlog,
+                                spawn=args.spawn,
+                                logger=logger,
+                                address_family=address_family,
+                                socket_type=_SOCKET_TYPE[args.socket_type])
+            try:
+                httpd.serve_forever()
+            except KeyboardInterrupt:
+                sys.exit(0)
+        finally:
+            if post_hook is not None:
+                logger.info('Running the post-hook %r' % post_hook)
+                post_hook(args)
+
+    if args.use_reloader:
+        try:
+            from werkzeug.serving import run_with_reloader
+        except ImportError:
+            logger.info("Reloader requires Werkzeug: "
+                        "'pip install werkzeug'")
             sys.exit(0)
-    finally:
-        if post_hook is not None:
-            logger.info('Running the post-hook %r' % post_hook)
-            post_hook(args)
+        run_with_reloader(inner)
+    else:
+        inner()
 
 
 if __name__ == '__main__':
