@@ -5,10 +5,12 @@ import socket
 import threading
 import select
 import random
-import Queue
+from six.moves import queue
 import logging
 import fcntl
 
+import six
+
 
 LOG_LEVELS = {
     "critical": logging.CRITICAL,
@@ -37,7 +39,7 @@ def configure_logger(logger, level='INFO', output="-"):
         close_on_exec(h.stream.fileno())
     fmt = logging.Formatter(LOG_FMT, LOG_DATE_FMT)
     h.setFormatter(fmt)
-    logger.addHandler(h)
+    logger.handlers = [h]
 
 
 class ImportStringError(ImportError):
@@ -100,8 +102,8 @@ def import_string(import_name, silent=False):
     :return: imported object
     """
     # force the import name to automatically convert to strings
-    if isinstance(import_name, unicode):
-        import_name = str(import_name)
+    if not six.PY3 and isinstance(import_name, unicode):
+        import_name = import_name.encode('utf-8')
     try:
         if ':' in import_name:
             module, obj = import_name.split(':', 1)
@@ -111,7 +113,7 @@ def import_string(import_name, silent=False):
             return __import__(import_name)
             # __import__ is not able to handle unicode strings in the fromlist
         # if the module is a package
-        if isinstance(obj, unicode):
+        if not six.PY3 and isinstance(obj, unicode):
             obj = obj.encode('utf-8')
         try:
             return getattr(__import__(module, None, None, [obj]), obj)
@@ -121,20 +123,21 @@ def import_string(import_name, silent=False):
             modname = module + '.' + obj
             __import__(modname)
             return sys.modules[modname]
-    except ImportError, e:
+    except ImportError as e:
         if not silent:
-            raise ImportStringError(import_name, e), None, sys.exc_info()[2]
+            six.reraise(ImportStringError, ImportStringError(import_name, e),
+                        sys.exc_info()[2])
 
 
 def hello_app(environ, start_response):
     status = '200 OK'
     headers = [('Content-type', 'text/plain')]
     start_response(status, headers)
-    return ['hello world']
+    return [b'hello world']
 
 
 _IN = _OUT = None
-_DBS = Queue.Queue()
+_DBS = queue.Queue()
 
 
 _ITEMS = """\
@@ -163,7 +166,7 @@ class _FakeDBThread(threading.Thread):
         while True:
             rl, __, __ = select.select([self.read2], [], [], 1.)
             if rl == []:
-                print 'nothing came back'
+                print('nothing came back')
                 continue
             current = os.read(self.read2, 1024)
             data.append(current)
@@ -244,7 +247,7 @@ def bench_app(environ, start_response):
     return result
 
 
-def create_socket(host, port, family=socket.AF_INET, type=socket.SOCK_STREAM,
+def create_socket(host, port=0, family=socket.AF_INET, type=socket.SOCK_STREAM,
                   backlog=2048, blocking=True):
     if family == socket.AF_UNIX and not host.startswith('unix:'):
         raise ValueError('Your host needs to have the unix:/path form')
@@ -256,8 +259,10 @@ def create_socket(host, port, family=socket.AF_INET, type=socket.SOCK_STREAM,
         # just recreate the socket
         fd = int(host.split('://')[1])
         sock = socket.fromfd(fd, family, type)
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     else:
         sock = socket.socket(family, type)
+        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         if host.startswith('unix:'):
             filename = host[len('unix:'):]
             try:
